//This file is "inversion.h"
//Header file for class Inversion
//By Teck Leong Tan

#ifndef _INVERSION_H
#define _INVERSION_H
#include <vector>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <cmath>
#include <string>
#include "clustertree.h"
#include "power.h"
#include "trans_vectors.h"
using namespace std;

/* The inversion class is written to do structural inversion to obtain the
   cluster interaction energy. Given a structural energies E and cluster 
   function X, we try to solve by linear least square to obtain the cluster
   interaction energy J.
   Therefore we are trying to solve E=XJ, where:
   E is a (ndat by 1) vector. In experimental terms, this is the data set.
   X is a (ndat by ma) matrix
   J is a (ma by 1 ) vector. J is the unknown to solve for.
 
   Structural Inversion is done in the following manner:
   1. E is read into the vector E[0...ndat-1]
   2. X is stored in phi[0...ndat-1][0...ma-1]
   3. The func. lfit() 1st create this equation X'E=X'XJ, where
      X' is the transpose of X. Hence, X'X is a square matrix.
      And X'E and J are both (ma by 1) vectors.
   4. gaussj() is called to perform gaussian elimination on the square 
      matrix X'X and X'E is reduced to J in the process too.
*/
//class Trans_vectors;

class Inversion
{
  public:
    friend class Trans_vectors;//The class Trans_vectors can use the members here
    Inversion();
    Inversion(int num);
    //vector<char*> sinv_filenames;//names of inputfiles for structural inversion.
      
    int gaussj(){gaussj(A,B);};//Helper function to call private gaussj
    //The above func does linear equation solution by Gauss-Jordan elimination
    //Refer to Numerical Recipe chapter2
    void lfit();
    //The above does general least square fit.
    //Refer to Numerical Recipe chapter15.4.
    void covsrt(vector<bool> &ia, const int mfit);
    //The above sort covariance matrix to orginal order
    //Used by lfit()

    void swap (double &a, double &b);//Swap 2 doubles
    void set_param();//Sets the parameters ia, sig and initialise covar
    //void set_param(const vector<int> &nc);//Sets paramters ia, sig using info from cluster indices
                                         //For choosing of optimal cluster expansion
    void cal_cv1(const vector<int> &nc, const ClusterTree &clustree);
    //The above calculates CV score for given clusters defined by nc 
    void print_matrix(char*);
    void print_outphi();//Print correlation matrix
    void invert_matrix(vector< vector<double> > &a);
    void cv1_slow();//Calculate CV -1 score the slow way
    void cv1_fast();//Calculate CV -1 score the 'smart' way

    vector<double> E; //Set of energy data (absolute energy), E.
    vector<double> J; //Set of initially unknown cluster interaction, J.
    vector< vector<double> > phi;//Matrix of cluster functions
    vector<string> sid;//structural id
    vector<double> cv1_score;//CV -1 score
    vector<double> corr_disorder;//Correlations for disordered states
    vector<int> n_index, f_index, type_index; //Assigned in prep_vin_&Inversion)
    //The above gives n,f,atomtype, indices for each element of J. 
    //vector<int> degen;//degeneracy of the cluster associated with each element of J.
    int num_types;//no. of atom types;
    char ensemble;//'g' for grand canonical. 'c' for canonical.
    //'c' is fixed composition and all 1-body terms are neglected

    double dot_product(const vector<double> &a, const vector<double>& b);

    //Some extra variables, which may be useful next time
    vector<double> Ef; //Formation energy
    vector<double> dE; //Error in energy
    double Edisorder;//Disorder energy 

    int issingular;//If the matrix is singular
       
  private:
    //Inversion();//Constructor not allowed
    int gaussj(vector< vector<double> > &a, vector< vector<double> >&b);

    //Ax=B where A is a 2D matrix, x and B are vectors
    //However, it is also possible for x & B to be augmented matrix, 
    //consisting of 1 to m vectors
    vector< vector<double> > A;//2D matrix
    vector< vector<double> > B;//2D matrix

    //double dot_product(const vector<double> &a, const vector<double>& b);
    //The following are parameters needed for lfit()
    vector< vector<double> > covar;//covariance matrix.Equal to inverse(phiT phi)
                                   //phiT is transpose of matrix phi
    vector <bool> ia; //Keep track of which data are actually fixed(fixed=0). 
                      //Will be set to 1.
    vector <double> sig; //Sigma. Error of data. Will be set to 1.  
    double chisq; // Chi-square value of data fit. 
};

#endif
//EOF

