/* This file is print_param.cc
   Member function of class Lattice
   By Teck Leong Tan
*/

#include "lattice.h"

/* This function prints out the parameters of the MC run */

bool Lattice::print_param()
{
  //Print to file param.out the inputs================================
  fstream param;
  param.open(outparam, ios::out);
  if (!param)
  {
    cout<<"File: "<<outparam<<" is not opened!"<<endl;
    exit(1);
  }
  param<<"No. of dimensions: " <<ndim<<endl;
  for (int i=0;i<ndim;i++)
  {
    param<<"No. of cells in direction"<<i+1<<": "<<nrep[i]<<endl;
  }
  param<<"No. of atoms per sublattice: "<<nsize<<endl;
  param<<"no. of sublattices= "<<num_sublat<<endl;
  param<<"No. of atom types: "<<num_type<<endl;
  param<<"Simulation type: "<<canonical<<endl;
  param<<"Type of lattice initialisation (restart option): "<<restart<<endl;
  param<<"Initial concentrations: ";
  for (int i=0;i<init_conc.size();i++)
  {
    param<<init_conc[i]<<" ";
  }
  param<<endl;
  
  /* Print to file the potentials & at the same time check that symmetric potential
     are equal. E.g V12=V21, V112=V121=V211. As potentials are stored in linear 
     format, checking is done in the following manner (e.g for pair interaction 
     with num_type=2, only 2 atomic types are considered as independent):
     1. In linear format, we have V[0], V[1], V[2], V[3]
     2. In tensor format, we have V[0,0], V[0,1], V[1,0] V[1,1]
     3. Tensor format stored in vector V_index of size 2 (=no. of bodies)
     4. V_factor vector stores the corresponding conversion factor for 
        each tensor indices.
        e.g V[2]=V[1,0] since 1*2+0*1=2. V[3]=V[1,1] since 1*2+1*1=3 
     5. V_tensor is sorted to give the lowest possible index e.g V[1,0] to V[0,1]
     6. Program then checks if V[1,0]=V[0,1] within a tolerance value. If not a
        a warning is printed to file.*/
 
  //Start of printing and checking of potentials------------------------------- 
  
  vector<int> V_tensor;//Tensor indexing of interaction 
  vector<int> V_factor;//For transforming from tensor indexing to linear indexing
  int v_linear; //Linear indexing of interaction
  int v_lowest; //Lowest possible linear index for a given tensor
  double lowest_v, linear_v; //Intermediate storage of potential

  param<<"Potentials are: "<<endl;

  for (int ll=0; ll<num_sublat;ll++)
  {
    for (int n=0;n<maxbody+1;n++)
    {
      V_factor.assign(n,1);
      V_tensor.assign(n,1);//Tensor format
      
      //Loop over n bodies backwards
      for (int i=0;i<=n-1;i++ )
      { 
          V_factor[i]*=power(num_type,(n-1-i));
      }//end for (int i=n-1;i>=0;i-- )

      for (int f=0;f<clustree.sublat[ll].nbody[n].nclus.size();f++)
      {
        param<<"Sub_lat "<<ll<<" ,nbody "<<n<<" ,cluster "<<f<<"  "<<endl;
        //V_factor.assign((power(Lat.num_type,);

        for (int v=0;v<clustree.sublat[ll].nbody[n].nclus[f]->Vin.size();v++)
        {
          v_linear=v;

          //Loop over no. of bodies
          for (int i=0;i<=n-1;i++ )
          {
            //Change from linear index to tensor index by successive division
            V_tensor[i]=v_linear/V_factor[i];
            v_linear=v_linear%V_factor[i];
            //cout <<"before sorting:"<<V_tensor[i];
          }

          sort(V_tensor.begin(),V_tensor.end());//Sort to obtain tensor with lowest value

          v_lowest=0; 
          for (int i=0; i<n;i++)
          {
            v_lowest+=(V_tensor[i]*V_factor[i]);
            //cout<< "after sorting:" <<V_tensor[i];
          }
          //cout<<"v_index = "<<v<<endl;
          //cout<<"v_lowest= "<<v_lowest<<endl;
          lowest_v=clustree.sublat[ll].nbody[n].nclus[f]->Vin[v_lowest];
          linear_v=clustree.sublat[ll].nbody[n].nclus[f]->Vin[v];

          param<<linear_v;
          //if potential is not symmetric
          if ( abs(linear_v-lowest_v)>0.000001)
          {
            param<<"Warning in print_param.cc: Check that V["<<v<<"] and V["<<v_lowest<<"]are equal";
            cout<<"Warning in print_param.cc: Check that V["<<v<<"] and V["<<v_lowest<<"]are equal";
            
          }
          param<<endl;
        }
        //param<<Lat.clustree.sublat[ll].nbody[n].nclus[f]->Vin.size();//extra
        //param<<Lat.clustree.sublat[ll].nbody[n].nclus.size();//extra
        param<<endl;
      }
    }  
  }
  //End of printing and checking potentials----------------------------------------

  param<<"No. of passes for MC: "<<numpass<<endl;
  param<<"No. of equilibration passes: "<<equil_pass<<endl;
  param<<"Random no. seed is: "<<randseed<<endl;
  //param<<"No. of passes per output: "<<write_freq<<endl;
  param<<"Initial, Final and Increment of Temperature are: ";
  param<<Tinit<<", "<<Tfinal<<", "<<Tincrement<<","<<endl; 
  for (int i=0;i<num_type;i++)
  {
    param<<"Init., Final and Increment of mu_s are: "<<endl;
    param<<"For type "<<i+1<<": ";
    param<<mu_init[i]<<", "<<mu_final[i]<<", ";
    param<<mu_increment[i]<<","<<endl;
  }
  //cout<<"size of Lat.mu_init= "<<sizeof(Lat.mu_init)<<endl;
  //Print elem_index
  param<<"Element and indices: "<<endl;
  for (int i=0; i<elem_index.size();i++)
  {
    param<<elem_index[i].element<<" "<<elem_index[i].index<<endl;
  }  
  
  if (correlation_option==0) param<<"Correlation disabled."<<endl;
  else
  { 
    param<<"Correlation enabled."<<endl;
    param<<"Pair option: "<<pair_option<<endl;
    if (pair_option==1)
    {
      param<<"Pair cutoff: "<<pair_cutoff<<endl;
    }
    param<<"LRO option: "<<LRO_option<<endl;
     
  }
  param<<endl;
  cout<<"end of printing to param"<<endl;
  //End of print out to param.out======================================

  //input.close();
  param.close();
}

//EOF
